<?php
/*
Script Name: planet RSS XML feed checker - the Ajax server part of it
Script URI: http://planetozh.com/blog/my-projects/planet-rss-xml-ajax-feed-checker/
Author: Ozh
Author URI: http://planetozh.com/
*/

// XML and header stuff
header('Content-Type: text/xml; charset=UTF-8');
header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
header("Cache-Control: no-cache, must-revalidate");
header("Pragma: no-cache");
echo '<?xml version="1.0" encoding="utf-8"?>';

define('VALID','Feed OK');
define('NOTVALID', 'Feed not found or empty');
define('FOUND', 'Feed found');


getResponse();

// Returns parts of a URL (scheme, host, path, port)
function getUrl($url) {
	// Default components of the URL
	$defaults = array(
		'scheme' => 'http',
		'host' => 'error-with-host.com',
		'port' => 80,
		'user' => '',
		'pass' => '',
		'path' => '/',
		'query' => '',
		'fragment' => ''); 

	// Parse input : the URL
	$args = parse_url($url);
	$args=array_merge($defaults,$args);
	$path = (substr($args['path'],0,1)=='/')?$args['path']:'/'.$args['path'];
	if ($args['query']) $path .= '?'.$args['query'];
	$url = $args['scheme'].'://'.$args['host'].$path;
	
	return array(
		'url'   => $url,
		'host'  =>	$args['host'],
		'port'  => $args['port'],
		'query' => $args['query'],
		'path'  => $path,
	);
	
}

// Query a server and listen to its response
function getSock($host, $port, $path) {
	$ua = 'FeedChecker 1.00 +http://planetozh.com/blog/my-projects/planet-rss-xml-feed-checker/';
	$query = "GET $path HTTP/1.0\nAccept: */*\nHost: $host\nUser-Agent: $ua\nConnection: Close\n\n";

	// Query server and listen to its response
	$fp = @fsockopen($host, $port, $errno, $errstr, 30);
	if(!$fp) {
		$result = false;
	} else {
		fputs($fp,$query);
		while(!feof($fp)) {
			$result .= fgets($fp,300);
		}
		fclose($fp);
		$result = preg_replace('|(</[^>]+>)|', "$1\n", $result);
		$result = explode("\n",$result);
	}
	
	return $result ;
}

// Main function : echo our XML result for Ajax parsing
function getResponse($url = '') {

	if ($url == '') $url = $_GET['url'];

	$args = getUrl($url);
	
	$result = getSock($args['host'],$args['port'],$args['path']);
	
	/* Response is something like the following, in 4 parts :
		- the HTTP Code (200 OK, 404 Not Found, etc)
		- a few server headers (Something: Somevalue)
		- a blank line
		- the content of the page itself
		Example :

		HTTP/1.1 200 OK
		Date: Fri, 11 May 2007 18:55:30 GMT
		Server: Apache/1.3.37 (Unix) mod_throttle/3.1.2 
		Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0
		ETag: "0e1da2b2f629539647ea6d6a84bd599a"
		Expires: Thu, 19 Nov 1981 08:52:00 GMT
		Pragma: no-cache
		
		<html>
		<body>
		...
	*/

	$days = 'No entry found';
	$type = 'Not XML';
	$pubDate = -1;

	if ($result !== false) {
		// Let's analyse the server's response
		
		// HTTP Code	
		$http = array_shift($result); // 'HTTP/1.1 200 OK'
		$http = trim(substr($http,strpos($http,' ')+1)); // '200 OK'
		if ($http == '200 OK') $http = 'Found';
		
		// HTTP headers, and content (HTML or XML or anything)
		$doheaders = true;
		foreach($result as $line) {
			$line = trim($line);
			if ($line == "") $doheaders = false;
			// until we encounter a blank line, we're still parsing headers.
			// After blank line we'll be parsing real content
			if ($doheaders) {
				$key = trim(substr($line,0,strpos($line,':')));
				$value = trim(substr($line,strpos($line,':')+1));
				if ($key == 'Location') {
					// we have something like Location: http://othersite.com/feed/
					// We're following redirections !
					getResponse($value);
					exit;
				}
				$headers[$key] = $value; // $header['Content-Type'] = 'text/xml'
			} else {
				$content[] = $line;
			}
		}
		
		// Does it smells like XML ? (it should)
		if (array_key_exists('Content-Type',$headers)) {
			if (strpos($headers['Content-Type'], 'xml') !== false)
				$type = 'XML';
		}
		
		// look for the first <pubDate> (RSS) or <published> (ATOM) entry to get its age
		$pubDate = 'not found';
		$found_item = false;
		$found_date = false;
		foreach ($content as $line) {
			if (
				(strpos($line,'<item>') !== false)
				or
				(strpos($line,'<entry>') !== false)
			) $found_item = true; // we found the first <item>
			if (
				(
					(strpos($line,'<pubDate>') !== false)
					or
					(strpos($line,'<published>') !== false)
				) && $found_item ) {
				$pubDate = $line;
				$pubDate = str_replace('<pubDate>','',$pubDate);
				$pubDate = str_replace('</pubDate>','',$pubDate);
				$pubDate = str_replace('<published>','',$pubDate);
				$pubDate = str_replace('</published>','',$pubDate);
				$found_date = true;
				break;
			}
		}
		
		if ($found_date) {
			$date = strtotime($pubDate);
			$now = time();
			$days = intval((($now-$date)/86400)*10)/10;
			$days = "Updated $days days ago";
		}

	} else {
		// Server's response was an error, or there was no response
		$url = 'host unreachable';		
		$http = 'Error';
	
	}
	
echo <<<XML
<result>
	<url>$url</url>
	<http>$http</http>
	<type>$type</type>
	<pubDate>$pubDate</pubDate>
	<days>$days</days>
</result>
XML;
	
}

?>